package org.freihoff.yawl.persistence.xmladapter;

import org.freihoff.yawl.persistence.generated.EmployeeType;
import org.freihoff.yawl.persistence.generated.ObjectFactory;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Properties;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Copyright 2014 Christian Freihoff
 * 
 * @author Christian Freihoff
 *
 * responsible for persisting and loading xml datasets to/from the database
 *
 * Note: only send xml which matches the XSD/JAXBElement Type
 */
public class EmployeeXmlAdapter {

    private JAXBContext context;
    private ObjectFactory objectFactory;
    private EntityManagerFactory entityManagerFactory;
    private static Log LOG = LogFactory.getLog(EmployeeType.class);
    
    public EmployeeXmlAdapter(){
        setUpContext();
        setPersistenceProperties();
    }

    public String read(Long hjid) {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);

        final Marshaller marshaller;
        OutputStream outputStream = new ByteArrayOutputStream();
        try {
            marshaller = context.createMarshaller();
            marshaller.marshal(objectFactory.createEmployee(employee), outputStream);
        } catch (JAXBException ex) {
            LOG.error("JAXBException while marshalling");
        }       
        String outputString = outputStream.toString();
     
        entityManager.close();
        
        return outputString;
    }
    
    public Long write(String employeeString) {
        final Object object;
        final EmployeeType employee;
        Long hjid = null;
        try {
            final Unmarshaller unmarshaller = context.createUnmarshaller();
            InputStream inputStream = new ByteArrayInputStream(employeeString.getBytes("UTF-8"));
            object = unmarshaller.unmarshal(inputStream);
            employee = ((JAXBElement<EmployeeType>) object).getValue();

            setPersistenceProperties();

            final EntityManager entityManager = entityManagerFactory.createEntityManager();

            entityManager.getTransaction().begin();
            entityManager.persist(employee);
            entityManager.getTransaction().commit();
            entityManager.close();

            hjid = employee.getHjid();

        } catch (JAXBException ex) {
            LOG.error("JAXB Exception while creating unmarshaller");
        } catch (UnsupportedEncodingException ex) {
            LOG.error("UnsupportedEncodingException while creating InputStream");
        }

        return hjid;
    }

    public void update(String employeeString, Long hjid) {
        final Object object;
        final EmployeeType employee;
        try {
            final Unmarshaller unmarshaller = context.createUnmarshaller();
            InputStream inputStream = new ByteArrayInputStream(employeeString.getBytes("UTF-8"));
            object = unmarshaller.unmarshal(inputStream);
            employee = ((JAXBElement<EmployeeType>) object).getValue();

            employee.setHjid(hjid);

            setPersistenceProperties();

            final EntityManager entityManager = entityManagerFactory.createEntityManager();

            entityManager.getTransaction().begin();
            entityManager.merge(employee);
            entityManager.getTransaction().commit();
            entityManager.close();

        } catch (JAXBException ex) {
            LOG.error("JAXB Exception while creating unmarshaller");
        } catch (UnsupportedEncodingException ex) {
            LOG.error("UnsupportedEncodingException while creating InputStream");
        }
    }
    
    public void delete(Long hjid){
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);
        entityManager.getTransaction().begin();
        entityManager.remove(employee);
        entityManager.getTransaction().commit();
        entityManager.close();
    }
    
    public void setBlocker(Long hjid, boolean value){
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);
        employee.setBlocked(value);
        entityManager.getTransaction().begin();
        entityManager.merge(employee);
        entityManager.getTransaction().commit();
        entityManager.close(); 
    }

    private void setUpContext() {
        try {
            context = JAXBContext.newInstance("org.freihoff.yawl.persistence.generated");
        } catch (JAXBException ex) {
            LOG.error("JAXBException, creating new JAXBContext failed");
        }
        objectFactory = new ObjectFactory();
    }

    private void setPersistenceProperties() {
        final Properties persistenceProperties = new Properties();
        InputStream inputStream;
        inputStream = getClass().getClassLoader().getResourceAsStream("persistence.properties");
        try {
            persistenceProperties.load(inputStream);
        } catch (IOException ioex) {
            LOG.error("loading persistence properties failed");
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException ignored) {
                }
            }
        }

        entityManagerFactory = Persistence.createEntityManagerFactory("org.freihoff.yawl.persistence.generated", persistenceProperties);

    }
    
    public void close(){
        entityManagerFactory.close();
    }
}
